/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

header
{
package pku.as.persistence.query.ejbql.antlr;

import java.util.ArrayList;
import java.util.List;

import pku.as.persistence.query.ejbql.tree.Node;
import pku.as.persistence.query.ejbql.tree.QueryNode;
import pku.as.persistence.query.ejbql.NodeFactory;
}

/**
 * EJB Query Language Grammar
 * This grammar parses the query language for EJB 3.0. 
 */
class EJBQLParser extends Parser;

options
{
	exportVocab=EJBQL;
	buildAST=false;
	k=3;    // For 'not like', 'not in', etc.
}

tokens
{
    ABS="abs";
    ALL="all";
    AND="and";
    ANY="any";
    AS="as";
    ASC="asc";
    AVG="avg";
    BETWEEN="between";
    BOTH="both";
    BY="by";
    CONCAT="concat";
    COUNT="count";
    CURRENT_DATE="current_date";
    CURRENT_TIME="current_time";
    CURRENT_TIMESTAMP="current_timestamp";
    DESC="desc";
    DELETE="delete";
    DISTINCT="distinct";
    EMPTY="empty";
    ESCAPE="escape";
    EXISTS="exists";
    FALSE="false";
    FETCH="fetch";
    FROM="from";
    GROUP="group";
    HAVING="having";
    IN="in";
    INNER="inner";
    IS="is";
    JOIN="join";
    LEADING="leading";
    LEFT="left";
    LENGTH="length";
    LIKE="like";
    LOCATE="locate";
    LOWER="lower";
    MAX="max";
    MEMBER="member";
    MIN="min";
    MOD="mod";
    NEW="new";
    NOT="not";
    NULL="null";
    OBJECT="object";
    OF="of";
    OR="or";
    ORDER="order";
    OUTER="outer";
    SELECT="select";
    SET="set";
    SIZE="size";
    SQRT="sqrt";
    SOME="some";
    SUBSTRING="substring";
    SUM="sum";
    TRAILING="trailing";
    TRIM="trim";
    TRUE="true";
    UNKNOWN="unknown";
    UPDATE="update";
    UPPER="upper";
    WHERE="where";
}

{
    /** Flag indicating whether aggregates are allowed. */
    private boolean aggregatesAllowed = false;
    
    private NodeFactory factory = NodeFactory.getInstance();
    
    private QueryNode root;
    
    public QueryNode getRoot() { return root; }

}

statement
    : root = selectStatement
    | root = updateStatement
    | root = deleteStatement
    ;
    
selectStatement returns [QueryNode node] { 
    node = null;
    Node select, from;
    Node where = null;
    Node groupBy = null;
    Node having = null;
    Node orderBy = null;
    List nodes = new ArrayList();
}
    : select = selectClause { nodes.add(select); }
      from = fromClause { nodes.add(from); }
      (where = whereClause { nodes.add(where); })?
      (groupBy = groupByClause { nodes.add(groupBy); })?
      (having = havingClause { nodes.add(having); })?
      (orderBy = orderByClause { nodes.add(orderBy); })?
      EOF {
      	node = (QueryNode)factory.create(NodeFactory.SELECTQUERY, nodes );
      }
    ;  
    
updateStatement returns [QueryNode node] { 
    node = null;
    Node update, set, where = null;
    List nodes = new ArrayList();
}

    : update = updateClause { nodes.add(update); }
      set = setClause { nodes.add(set); }
      (where = whereClause { nodes.add(where); })?
      EOF {	
        node = (QueryNode)factory.create(NodeFactory.UPDATEQUERY, nodes );
	  }
    ;
    
deleteStatement returns [QueryNode node]
{ 
    node = null; 
    Node delete, where = null;
    List nodes = new ArrayList();
}
    : delete = deleteClause { nodes.add(delete); }
      (where = whereClause { nodes.add(where); })?
      EOF {  	
        node = (QueryNode)factory.create(NodeFactory.DELETEQUERY, nodes );
	  }
    ;
    
fromClause returns [Node node]
{ 
    node = null;
    Node idVar, collMem = null;
    List nodes = new ArrayList(); 
}
    : t:FROM idVar = identificationVariableDeclaration  { nodes.add(idVar); }
        (COMMA  ( idVar = identificationVariableDeclaration  { nodes.add(idVar); }
                | collMem = collectionMemberDeclaration  { nodes.add(collMem); }
                ) 
        )*
        { node = factory.create(NodeFactory.FROM, nodes ); }
    ;
    
identificationVariableDeclaration returns [Node node]
{ 
	node = null; 
	Node rangeVar, joins = null;
	List nodes = new ArrayList(); 
}
    : rangeVar = rangeVariableDeclaration { nodes.add(rangeVar); } 
        ( joins = join { nodes.add(joins); } 
        )*
        { node = factory.create(NodeFactory.IDVARDECL, nodes ); }
    ;

rangeVariableDeclaration returns [Node node]
{ 
    node = null; 
    String schema;
    List nodes = new ArrayList(); 
}
    : schema = abstractSchemaName { nodes.add(schema); } 
    	(AS)? i:IDENT { nodes.add(i.getText()); }
        { 
            node = factory.create(NodeFactory.RANGEVARDECL, nodes ); 
        }
    ;
    
abstractSchemaName returns [String schema]
{ schema = null; }
    : ident:. 
        {
            schema = ident.getText();
        }
    ;
    
join returns [Node node]
{ 
    node = null;
    boolean outerJoin; 
    Node pathExpr;
    List nodes = new ArrayList();
}
    : outerJoin = joinSpec { nodes.add(outerJoin); }
      ( pathExpr = pathExpression { nodes.add(pathExpr); }
      	(AS)? i:IDENT { nodes.add(i.getText()); }
        {
            node = factory.create(NodeFactory.JOIN, nodes ); 
        }
      | t:FETCH pathExpr = pathExpression { nodes.add(pathExpr); }
        { 
            node = factory.create(NodeFactory.FETCHJOIN, nodes );
        }
      )
    ;
    
joinSpec returns [boolean outer]
{ outer = false; }
    : (LEFT (OUTER)? { outer = true; }  | INNER  )? JOIN
    ;
    
pathExpression returns [Node node]
{ 
    node = null; 
    List nodes = new ArrayList(); 
}
    : idVar:IDENT { nodes.add(idVar.getText()); }
        (d:DOT field:IDENT { nodes.add(field.getText()); }
        )+
        { node = factory.create(NodeFactory.PATHEXPR, nodes ); }
    ;

identificationVariable returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
	
}
    : i:IDENT
        { 
        	nodes.add(i.getText());
        	node = factory.create(NodeFactory.IDVAR, nodes ); 
        }
    ;

collectionMemberDeclaration returns [Node node]
{ 
	node = null; 
	Node pathExpr = null;
	List nodes = new ArrayList(); 
}
    : t:IN LEFT_ROUND_BRACKET 
    	pathExpr = pathExpression { nodes.add(pathExpr); }
    	RIGHT_ROUND_BRACKET 
      	(AS)? i:IDENT { nodes.add(i.getText()); }
        {
        	node = factory.create(NodeFactory.COLLECTIONMEMBERDECL, nodes ); 
        }
    ;
    
updateClause returns [Node node]
{ 
    node = null; 
    String schema = null;
    List nodes = new ArrayList(); 
}
    : u:UPDATE schema = abstractSchemaName { nodes.add(schema); }
        ((AS)? i:IDENT { nodes.add(i.getText()); })?
        { 
            node = factory.create(NodeFactory.UPDATE, nodes ); 
        } 
    ;  

setClause returns [Node node]
{ 
    node = null; 
    Node item;
    List nodes = new ArrayList();
}
    : t:SET item = updateItem { nodes.add(item); }
        (COMMA item = updateItem { nodes.add(item); } )*
        { 
        	node = factory.create(NodeFactory.SET, nodes );  
        }
     ;

updateItem returns [Node node]
{ 
    node = null;
    Node pathExpr, value; 
    List nodes = new ArrayList();
}
    : pathExpr = pathExpression { nodes.add(pathExpr); }
    	t:EQUALS 
    	value = newValue { nodes.add(value); }
        { 
            node = factory.create(NodeFactory.UPDATEITEM, nodes );
        }
    ;

newValue returns [Node node]
{ node = null; }
    : node = simpleArithmeticExpression
    ;

deleteClause returns [Node node]
{ 
    node = null; 
    String schema = null;
    List nodes = new ArrayList();
}
    : t:DELETE FROM schema = abstractSchemaName { nodes.add(schema); }
        ((AS)? 
        i:IDENT { nodes.add(i.getText()); })?
        { 
            node = factory.create(NodeFactory.DELETE, nodes );
        }
    ;

selectClause returns [Node node]
{ 
    node = null;
    boolean distinct = false;
    Node selectExpr;
    List nodes = new ArrayList();
}
    : t:SELECT (DISTINCT { distinct = true; })?  { nodes.add(distinct); }
      selectExpr = selectExpression { nodes.add(selectExpr); }
      ( COMMA selectExpr = selectExpression  { nodes.add(selectExpr); } )*
        { 
            node = factory.create(NodeFactory.SELECT, nodes );
        }
    ;
    
selectExpression returns [Node node]
{ 
	node = null; 
}
    : node = pathExpression
    | node = aggregateExpression
    | node = identificationVariable
    | node = objectExpression
    | node = constructorExpression
    ;
    
objectExpression returns [Node node]
{ 
    node = null;
    Node idVar;
    List nodes = new ArrayList();
}
    : t:OBJECT
        LEFT_ROUND_BRACKET 
        idVar = identificationVariable { nodes.add(idVar); } 
        RIGHT_ROUND_BRACKET
        { 
            node = factory.create(NodeFactory.OBJECTEXPR, nodes );; 
        }
    ;
    
constructorExpression returns [Node node]
{ 
    node = null;
    String className = null; 
    Node item;
    List nodes = new ArrayList();
}
    : t:NEW className = constructorName { nodes.add(className); }
        LEFT_ROUND_BRACKET 
        item = constructorItem { nodes.add(item); } 
        ( COMMA item = constructorItem { nodes.add(item); } )*
        RIGHT_ROUND_BRACKET
        { 
            node = factory.create(NodeFactory.SELECT, nodes );; 
        }
    ;

constructorName returns [String className]
{ 
    className = null;
    StringBuffer buf = new StringBuffer(); 
}
    : i1:IDENT { buf.append(i1.getText()); }
        ( DOT i2:IDENT { buf.append('.').append(i2.getText()); })*
        { className = buf.toString(); }
    ;

constructorItem returns [Node node]
{ node = null; }
    : node = pathExpression
    | node = aggregateExpression
    ;
    
aggregateExpression returns [Node node]
{ 
    node = null; 
    String operator = null;
    boolean distinct = false;
    Node expr;
    List nodes = new ArrayList();
}
    : (t1:AVG { operator = t1.getText(); } LEFT_ROUND_BRACKET (DISTINCT { distinct = true; })?
        expr = pathExpression RIGHT_ROUND_BRACKET 
    | t2:MAX { operator = t2.getText(); } LEFT_ROUND_BRACKET (DISTINCT { distinct = true; })? 
        expr = pathExpression RIGHT_ROUND_BRACKET
    | t3:MIN { operator = t3.getText(); } LEFT_ROUND_BRACKET (DISTINCT { distinct = true; })?
        expr = pathExpression RIGHT_ROUND_BRACKET
    | t4:SUM { operator = t4.getText(); } LEFT_ROUND_BRACKET (DISTINCT { distinct = true; })?
        expr = pathExpression RIGHT_ROUND_BRACKET
    | t5:COUNT { operator = t5.getText(); } LEFT_ROUND_BRACKET (DISTINCT { distinct = true; })?
        ( expr = identificationVariable
        | expr = pathExpression 
        ) RIGHT_ROUND_BRACKET
    )	    
    
    { 
    	nodes.add(operator);
    	nodes.add(distinct);
    	nodes.add(expr);
    	node = factory.create(NodeFactory.AGGREGATE, nodes ); 
    }
    ;
    
whereClause returns [Node node]
{ 
	node = null; 
	Node condition;
	List nodes = new ArrayList();
}
    : t:WHERE condition = conditionalExpression { nodes.add(condition); }
        { node = factory.create(NodeFactory.WHERE, nodes ); } 
    ;
    
groupByClause returns [Node node]
{ 
    node = null; 
    Node item;
    List nodes = new ArrayList();
}
    : g:GROUP BY
        item = groupByItem { nodes.add(item); }
        (COMMA item = groupByItem  { nodes.add(item); } )*
        { node = factory.create(NodeFactory.GROUPBY, nodes ); }
    ;
    
groupByItem returns [Node node]
{ 
	node = null; 
}
    : node = pathExpression
    | node = identificationVariable
    ;

havingClause returns [Node node]
{ 
	node = null; 
	Node condition;
	List nodes = new ArrayList();
}
    : h:HAVING { aggregatesAllowed = true; } 
        condition = conditionalExpression { nodes.add(condition); }
        { 
            aggregatesAllowed = false; 
            node = factory.create(NodeFactory.HAVING, nodes );
        }
    ;

orderByClause returns [Node node]
{ 
    node = null; 
    Node item;
    List nodes = new ArrayList();
}
    : o:ORDER BY
        item = orderByItem  { nodes.add(item); } 
        (COMMA item = orderByItem  { nodes.add(item); })*
        { node = factory.create(NodeFactory.ORDERBY, nodes ); }
    ; 

orderByItem returns [Node node]
{ 
	node = null;
	Node pathExpr;
	List nodes = new ArrayList(); 
}
    : pathExpr = pathExpression { nodes.add(pathExpr); }
        ( a:ASC { nodes.add(a.getText()); }
        | d:DESC { nodes.add(d.getText()); }
        | // empty rule
        )
        { node = factory.create(NodeFactory.ORDERBYITEM, nodes ); }
    ;

subquery returns [Node node]
{ 
    node = null; 
    Node select, from;
    Node where = null;
    Node groupBy = null;
    Node having = null;
    List nodes = new ArrayList(); 
}
	: select = simpleSelectClause { nodes.add(select); }
      from = subqueryFromClause { nodes.add(from); }
      (where = whereClause { nodes.add(where); })?
      (groupBy = groupByClause { nodes.add(groupBy); })?
      (having = havingClause { nodes.add(having); })?
        { 
            node = factory.create(NodeFactory.SUBQUERY, nodes ); 
        }
    ;

subqueryFromClause returns [Node node]
{ 
    node = null; 
    Node subIdVar = null;
    List nodes = new ArrayList(); 
}
    : f:FROM subIdVar = subselectIdentificationVariableDeclaration { nodes.add(subIdVar); } 
        ( COMMA subIdVar = subselectIdentificationVariableDeclaration { nodes.add(subIdVar); } )*
        { node = factory.create(NodeFactory.FROM, nodes ); }
    ;

subselectIdentificationVariableDeclaration returns [Node node]
{ 
	node = null;
	Node pathExpr;
	List nodes = new ArrayList(); 
}
    : node = identificationVariableDeclaration
    | pathExpr = pathExpression (AS)? ident:IDENT
        { 
        	nodes.add(pathExpr);
        	nodes.add(ident.getText());
        	node =  factory.create(NodeFactory.SUBIDVARDECL, nodes );
        }
    | node = collectionMemberDeclaration
    ;

simpleSelectClause returns [Node node]
{ 
    node = null; 
    boolean distinct = false;
    Node simpleSelectExpr;
    List nodes = new ArrayList();
}
    : s:SELECT (DISTINCT { distinct = true; nodes.add(distinct); })?
      simpleSelectExpr = simpleSelectExpression
        {
            nodes.add(simpleSelectExpr);
            node = factory.create(NodeFactory.SELECT, nodes );
        }
    ;

simpleSelectExpression returns [Node node]
{ 
	node = null; 
}
    : node = pathExpression
    | node = aggregateExpression
    | node = identificationVariable
    ;

conditionalExpression returns [Node node]
{ 
    node = null;
    Node term;
    List nodes = new ArrayList();
}
    : term = conditionalTerm { nodes.add(term); }
        (t:OR term = conditionalTerm { nodes.add(term); }
        )*
        { node = factory.create(NodeFactory.CONDITIONALEXPR, nodes ); }
    ;

conditionalTerm returns [Node node]
{ 
    node = null; 
    Node factor;
    List nodes = new ArrayList();
}
    : factor = conditionalFactor { nodes.add(factor); } 
        (t:AND factor = conditionalFactor { nodes.add(factor); }
        )*
        { node = factory.create(NodeFactory.CONDITIONALTERM, nodes ); }
    ;

conditionalFactor returns [Node node]
{ 
    node = null;
    boolean not = false;
    Node primary;
    List nodes = new ArrayList();
}
    : (n:NOT { not = true; })?  { nodes.add(not); } 
        primary = conditionalPrimary { nodes.add(primary); } 
        { 
            node = factory.create(NodeFactory.CONDITIONALFACTOR, nodes );
        }
    ;

conditionalPrimary  returns [Node node]
{ 
	node = null; 
}
    : (LEFT_ROUND_BRACKET conditionalExpression) =>
        LEFT_ROUND_BRACKET node = conditionalExpression RIGHT_ROUND_BRACKET
    | node = simpleConditionalExpression 
    ;

simpleConditionalExpression returns [Node node]
{ 
    node = null; 
    Node left = null;
}
    : left = arithmeticExpression 
        node = simpleConditionalExpressionRemainder[left]
    | node = existsExpression
    ;

simpleConditionalExpressionRemainder [Node left] returns [Node node]
{
	node = null; 
}
    : node = comparisonExpression[left]
    | ((NOT)? BETWEEN) => node = betweenExpression[left]
    | ((NOT)? LIKE) => node = likeExpression[left]
    | ((NOT)? IN) => node = inExpression[left]
    | (IS (NOT)? NULL) => node = nullComparisonExpression[left]
    | node = emptyCollectionComparisonExpression[left]
    | node = collectionMemberExpression[left]
    ;

betweenExpression [Node left] returns [Node node]
{ 
    node = null; 
    Node lower, upper;
    List nodes = new ArrayList();
}
    : ( n:NOT { nodes.add(n.getText()); })? 
    	t:BETWEEN 
        lower = arithmeticExpression AND upper = arithmeticExpression 
        { 
        	nodes.add(lower);
        	nodes.add(upper);
        	nodes.add(left);
            node = factory.create(NodeFactory.BETWEENEXPR, nodes );
        }
    ;

inExpression [Node left] returns [Node node]
{ 
    node = null;
    List nodes = new ArrayList();
    Node subqr, item; 
}
    : ( n:NOT )? t:IN 
        LEFT_ROUND_BRACKET
        ( item = inItem { nodes.add(item); } 
            ( COMMA item = inItem { nodes.add(item); } )*
        | subqr = subquery { nodes.add(subqr); }
        )
        RIGHT_ROUND_BRACKET
        {
        	nodes.add(left);
            node = factory.create(NodeFactory.BETWEENEXPR, nodes );
        }
    ;

inItem returns [Node node]
{ 
	node = null; 
}
    : node = literal
    | node = inputParameter
    ;

literal returns [Node node]
{ node = null; }
    : node = literalNumeric
    | node = literalBoolean
    | node = literalString
    ;

literalNumeric returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : i:NUM_INT 
    	{ 
    		nodes.add(i.getText());
        	node = factory.create(NodeFactory.LITERAL, nodes );
        }
    | f:NUM_FLOAT 
    	{ 
    		nodes.add(f.getText()); 
        	node = factory.create(NodeFactory.LITERAL, nodes );
    	}
    ;

literalBoolean returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : t:TRUE
    	{ 
    		nodes.add(t.getText()); 
        	node = factory.create(NodeFactory.LITERAL, nodes );
    	}
    | f:FALSE  
    	{ 
    		nodes.add(f.getText());
        	node = factory.create(NodeFactory.LITERAL, nodes );
        }
    ;

literalString returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : d:STRING_LITERAL_DOUBLE_QUOTED
    	{
    		nodes.add(d.getText()); 
            node = factory.create(NodeFactory.LITERAL, nodes );
        }
    | s:STRING_LITERAL_SINGLE_QUOTED
        { 
        	nodes.add(s.getText()); 
            node = factory.create(NodeFactory.LITERAL, nodes );
        }
    ;
    
    
likeExpression [Node left] returns [Node node]
{ 
    node = null; 
    boolean notlike = false;
    Node pattern, esc = null;
    List nodes = new ArrayList();
}
    : ( n:NOT { notlike = true; } )? { nodes.add(notlike); }
    	t:LIKE pattern = patternValue { nodes.add(pattern); }
        (esc = escape { nodes.add(esc); })?
        { 
        	nodes.add(left);
            node = factory.create(NodeFactory.LIKEEXPR, nodes );
        }
    ;

patternValue returns [Node node]
{ 
	node = null; 
}
    : node = literalString 
    | node = inputParameter
    ;

escape returns  [Node node]
{ 
	node = null; 
	Node literal;
	List nodes = new ArrayList();
}
    : t:ESCAPE { nodes.add(t.getText()); } literal = literalString { nodes.add(literal); }
        { node = factory.create(NodeFactory.ESCAPE, nodes ); }
    ;

nullComparisonExpression [Node left] returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : t:IS ( n:NOT { nodes.add(n.getText()); } )? NULL
        { 
        	nodes.add(left);
        	node = factory.create(NodeFactory.ESCAPE, nodes ); 
        }
    ;

emptyCollectionComparisonExpression [Node left] returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : t:IS ( n:NOT { nodes.add(n.getText()); } )? EMPTY
        { 
        	nodes.add(left);
        	node = factory.create(NodeFactory.ISEMPTY, nodes ); 
        }
    ;

collectionMemberExpression [Node left] returns [Node node]
{ 
	node = null; 
	Node pathExpr;
	List nodes = new ArrayList();
}
    : ( n:NOT )? t:MEMBER (OF)? pathExpr = pathExpression
        {
        	nodes.add(pathExpr);
            nodes.add(left);
        	node = factory.create(NodeFactory.COLLECTIONMEMBEREXPR, nodes ); 
        }
    ;

existsExpression returns [Node node]
{ 
	node = null; 
	Node subqr;
	List nodes = new ArrayList();
}
    : t:EXISTS LEFT_ROUND_BRACKET subqr = subquery RIGHT_ROUND_BRACKET
        { 
            nodes.add(subqr);
        	node = factory.create(NodeFactory.EXISTSEXPR, nodes );
        }
    ;

anyOrAllExpression returns [Node node]
{ 
	node = null;
	Node subqr;
	List nodes = new ArrayList();
}
    : a:ALL LEFT_ROUND_BRACKET subqr = subquery RIGHT_ROUND_BRACKET
        { 
        	nodes.add(a.getText());
        	nodes.add(subqr);
        	node = factory.create(NodeFactory.ANYALLEXPR, nodes );
        }
    | y:ANY LEFT_ROUND_BRACKET subqr = subquery RIGHT_ROUND_BRACKET
        { 
        	nodes.add(y.getText());
        	nodes.add(subqr);
        	node = factory.create(NodeFactory.ANYALLEXPR, nodes );
        }
    | s:SOME LEFT_ROUND_BRACKET subqr = subquery RIGHT_ROUND_BRACKET
        { 
        	nodes.add(s.getText());
        	nodes.add(subqr);
        	node = factory.create(NodeFactory.ANYALLEXPR, nodes );
        }
    ;

comparisonExpression [Node left] returns [Node node]
{ 
	node = null; 
	Node right;
	List nodes = new ArrayList();
}
    : t1:EQUALS right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t1.getText());
        	nodes.add(right); 
        	nodes.add(left); 
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    | t2:NOT_EQUAL_TO right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t2.getText());
        	nodes.add(right); 
        	nodes.add(left); 
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    | t3:GREATER_THAN right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t3.getText());
        	nodes.add(right); 
        	nodes.add(left); 
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    | t4:GREATER_THAN_EQUAL_TO right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t4.getText());
        	nodes.add(right); 
        	nodes.add(left); 
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    | t5:LESS_THAN right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t5.getText());
        	nodes.add(right);
        	nodes.add(left);  
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    | t6:LESS_THAN_EQUAL_TO right = comparisonExpressionRightOperand 
        { 
        	nodes.add(t6.getText());
        	nodes.add(right); 
        	nodes.add(left); 
        	node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
        }
    ;

comparisonExpressionRightOperand returns [Node node]
{ 
	node = null; 
}
    : node = arithmeticExpression 
    | node = anyOrAllExpression
    ;   
    
arithmeticExpression returns [Node node]
{ 
	node = null; 
}
    : node = simpleArithmeticExpression
    | LEFT_ROUND_BRACKET node = subquery RIGHT_ROUND_BRACKET
    ;
    
simpleArithmeticExpression returns [Node node]
{ 
    node = null; 
    Node left, right;
    List nodes = new ArrayList();
}
    : left = arithmeticTerm { nodes.add(left); }
        ( p:PLUS right = arithmeticTerm 
            { nodes.add(p.getText()); nodes.add(right); }
        | m:MINUS right = arithmeticTerm
        	{ nodes.add(p.getText()); nodes.add(right); }
        )* 
        {
        	node = factory.create(NodeFactory.ARITHMETICEXPR, nodes );
        }
    ;
    
arithmeticTerm  returns [Node node]
{ 
    node = null; 
    Node left, right;
    List nodes = new ArrayList();
}
    : left = arithmeticFactor { nodes.add(left); } 
        ( m:MULTIPLY right = arithmeticFactor 
            { nodes.add(m.getText()); nodes.add(right); }
        | d:DIVIDE right = arithmeticFactor
            { nodes.add(d.getText()); nodes.add(right); }
        )* 
        {
        	node = factory.create(NodeFactory.ARITHMETICTERM, nodes );
        }
    ;

arithmeticFactor returns [Node node]
{ 
    node = null; 
    Node right;
    List nodes = new ArrayList();
}
    : p:PLUS  right = arithmeticPrimary 
        { 
        	nodes.add(p.getText()); 
        	nodes.add(right); 
        	node = factory.create(NodeFactory.ARITHMETICFACTOR, nodes );
        }
    | m:MINUS right = arithmeticPrimary  
        { 
        	nodes.add(m.getText()); 
        	nodes.add(right); 
        	node = factory.create(NodeFactory.ARITHMETICFACTOR, nodes );
        }
    | node = arithmeticPrimary 
    ;
    
arithmeticPrimary returns [Node node]
{ 
	node = null; 
}
	: node = pathExpression
    | node = literalNumeric
    | LEFT_ROUND_BRACKET node = arithmeticExpression RIGHT_ROUND_BRACKET
    | node = inputParameter
    | node = functionsReturningNumerics
    | { aggregatesAllowed }? node = aggregateExpression
    | node = stringExpression
    | node = entityExpression
    ;

stringExpression returns [Node node]
{ 
	node = null; 
}    
    : node = stringPrimary
    | LEFT_ROUND_BRACKET node = subquery RIGHT_ROUND_BRACKET
    ;

stringPrimary returns [Node node]
{ 
	node = null; 
}
    : node = pathExpression
    | node = literalString 
    | node = inputParameter
    | node = functionsReturningStrings
    | { aggregatesAllowed }? node = aggregateExpression
    ;
    
entityExpression returns [Node node]
{ 
	node = null; 
}
    : node = pathExpression
    | node = inputParameter
    ;

functionsReturningNumerics returns [Node node]
{ 
	node = null; 
	Node expr, str, left, right, pattern, startPos = null;
	List nodes = new ArrayList();
}
    : a:ABS LEFT_ROUND_BRACKET expr = simpleArithmeticExpression RIGHT_ROUND_BRACKET
        { 
        	nodes.add(expr);
        	node = factory.create(NodeFactory.ABS, nodes );
        }
    | l:LENGTH LEFT_ROUND_BRACKET str = stringPrimary RIGHT_ROUND_BRACKET
        { 
        	nodes.add(str);
        	node = factory.create(NodeFactory.LENGTH, nodes );
        }
    | m:MOD LEFT_ROUND_BRACKET
        left = simpleArithmeticExpression COMMA 
        right = simpleArithmeticExpression
        RIGHT_ROUND_BRACKET
        { 
        	nodes.add(left);
        	nodes.add(right);
        	node = factory.create(NodeFactory.MOD, nodes );
        }  
    | s:SQRT 
        LEFT_ROUND_BRACKET expr = simpleArithmeticExpression RIGHT_ROUND_BRACKET
        { 
        	nodes.add(expr);
        	node = factory.create(NodeFactory.SQRT, nodes );
        }
    |  lo:LOCATE
        LEFT_ROUND_BRACKET 
        pattern = stringPrimary COMMA str = stringPrimary
        ( COMMA startPos = simpleArithmeticExpression )?
        RIGHT_ROUND_BRACKET
        { 
        	nodes.add(pattern);
        	nodes.add(str);
        	nodes.add(startPos);
        	node = factory.create(NodeFactory.LOCATE, nodes );
        }
    | si:SIZE 
        LEFT_ROUND_BRACKET expr = pathExpression RIGHT_ROUND_BRACKET
        { 
        	nodes.add(expr);
        	node = factory.create(NodeFactory.SIZE, nodes );
        }
    ;

functionsReturningDatetime returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : d:CURRENT_DATE 
        { 
        	nodes.add(d.getText()); 
        	node = factory.create(NodeFactory.DATETIME, nodes );
        }
    | t:CURRENT_TIME
        { 
        	nodes.add(t.getText()); 
        	node = factory.create(NodeFactory.DATETIME, nodes );
        }
    | ts:CURRENT_TIMESTAMP
        { 
        	nodes.add(ts.getText()); 
        	node = factory.create(NodeFactory.DATETIME, nodes );
        }
    ;

functionsReturningStrings returns [Node node]
{ 
	node = null; 
	Node firstArg, secondArg, string, start, length;
	Node trimChar = null;
	String trimSpec = null;
	List nodes = new ArrayList();
}
    : c:CONCAT 
        LEFT_ROUND_BRACKET 
        firstArg = stringPrimary COMMA secondArg = stringPrimary
        RIGHT_ROUND_BRACKET
        { 
        	nodes.add(firstArg);
        	nodes.add(secondArg);
        	node = factory.create(NodeFactory.CONCAT, nodes );
        }
    | s:SUBSTRING   
        LEFT_ROUND_BRACKET
        string = stringPrimary COMMA
        start = simpleArithmeticExpression COMMA
        length = simpleArithmeticExpression
        RIGHT_ROUND_BRACKET
        { 
        	nodes.add(string);
        	nodes.add(start);
        	nodes.add(length);
        	node = factory.create(NodeFactory.SUBSTRING, nodes );
        }
    | t:TRIM
        LEFT_ROUND_BRACKET 
        ( ( trimSpecification trimCharacter FROM )=> 
        trimSpec = trimSpecification trimChar = trimCharacter FROM )? 
        string = stringPrimary
        RIGHT_ROUND_BRACKET
        { 
        	nodes.add(trimSpec);
        	nodes.add(trimChar);
        	nodes.add(string);
        	node = factory.create(NodeFactory.TRIM, nodes );
        }
    | u:UPPER LEFT_ROUND_BRACKET string = stringPrimary RIGHT_ROUND_BRACKET
        { 
        	nodes.add(string);
        	node = factory.create(NodeFactory.UPPER, nodes );
        }
    | l:LOWER LEFT_ROUND_BRACKET string = stringPrimary RIGHT_ROUND_BRACKET
        { 
        	nodes.add(string);
        	node = factory.create(NodeFactory.LOWER, nodes );
        }
    ;
    
trimSpecification returns [String trimSpec]
{ 
	trimSpec = "BOTH"; 
}
    : LEADING
        { trimSpec = "LEADING"; }
    | TRAILING
        { trimSpec = "TRAILING"; }
    | BOTH
        { trimSpec = "BOTH"; }
    | // empty rule
    ;

trimCharacter returns [Node node]
{ 
	node = null; 
}
    : node = literalString
    | node = inputParameter
    | // empty rule
    ;
    
inputParameter returns [Node node]
{ 
	node = null; 
	List nodes = new ArrayList();
}
    : p:POSITIONAL_PARAM
        { 
            String text = p.getText().substring(1);
            nodes.add(text);
            node = factory.create(NodeFactory.POSITIONALPARAMETER, nodes );
        }
    | n:NAMED_PARAM
        { 
            String text = n.getText().substring(1);
            nodes.add(text);
            node = factory.create(NodeFactory.NAMEDPARAMETER, nodes );
        }
    ;
    
class EJBQLLexer extends Lexer;

options {
	exportVocab=EJBQL;      // call the vocabulary "Ejbql"
	k=2; // needed for newline, and to distinguish '>' from '>='.
	charVocabulary='\u0000'..'\uFFFE';	// Allow any char but \uFFFF (16 bit -1, ANTLR's EOF character)
	caseSensitive = false;
	caseSensitiveLiterals = false;
}
   
// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
    :   ('0'..'9'|'a'..'f')
    ;

WS  : (' ' | '\t' | '\n' | '\r')+
    { $setType(Token.SKIP); } ;

LEFT_ROUND_BRACKET
    : '('
    ;

RIGHT_ROUND_BRACKET
    : ')'
    ;

COMMA
    : ','
    ;

IDENT 
    : TEXTCHAR
    ;

protected
TEXTCHAR
    : ('a'..'z' | '_' | '$') ('a'..'z' | '_' | '$' | '0'..'9')*
    ;

// a numeric literal
NUM_INT
    {boolean isDecimal=false;}
    :   '.' {_ttype = DOT;}
            (('0'..'9')+ (EXPONENT)? (FLOAT_SUFFIX)? { _ttype = NUM_FLOAT; })?
    |   (   '0' {isDecimal = true;} // special case for just '0'
            (   ('x')
                (                                           // hex
                    // the 'e'|'E' and float suffix stuff look
                    // like hex digits, hence the (...)+ doesn't
                    // know when to stop: ambig.  ANTLR resolves
                    // it correctly by matching immediately.  It
                    // is therefor ok to hush warning.
                    options {
                        warnWhenFollowAmbig=false;
                    }
                :   HEX_DIGIT
                )+
            |   ('0'..'7')+                                 // octal
            )?
        |   ('1'..'9') ('0'..'9')*  {isDecimal=true;}       // non-zero decimal
        )
        (   ('l')
        
        // only check to see if it's a float if looks like decimal so far
        |   {isDecimal}?
            (   '.' ('0'..'9')* (EXPONENT)? (FLOAT_SUFFIX)?
            |   EXPONENT (FLOAT_SUFFIX)?
            |   FLOAT_SUFFIX
            )
            { _ttype = NUM_FLOAT; }
        )?
    ;

// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
    :   ('e') ('+'|'-')? ('0'..'9')+
    ;


protected
FLOAT_SUFFIX
    :   'f'|'d'
    ;

EQUALS
    : '='
    ;

GREATER_THAN
    : '>'
    ;

GREATER_THAN_EQUAL_TO
    : ">="
    ;

LESS_THAN
    : '<'
    ;

LESS_THAN_EQUAL_TO
    : "<="
    ;

NOT_EQUAL_TO
    : "<>"
    ;

MULTIPLY
    : "*"
    ;

DIVIDE
    : "/"
    ;

PLUS
    : "+"
    ;

MINUS
    : "-"
    ;

POSITIONAL_PARAM
    : "?" NUM_INT
    ;

NAMED_PARAM
    : ":" TEXTCHAR
    ;

// Added Jan 9, 2001 JED
// string literals
STRING_LITERAL_DOUBLE_QUOTED
    : '"' (ESC|~('"'|'\\'))* '"'
    ;

STRING_LITERAL_SINGLE_QUOTED
    : ('\'' '\\' '\'') => ('\'' '\\' '\'')
    | '\'' (ESC|~('\''|'\\') | ("''"))* '\'' 
    ;

// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
protected
ESC
    :   '\\'
        (   '_'
        |   'n'
        |   'r'
        |   't'
        |   'b'
        |   'f'
        |   '"'
        |   '\''
        |   '\\'
        |   ('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
        |   ('0'..'3')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :   ('0'..'7')
                (   
                    options {
                        warnWhenFollowAmbig = false;
                    }
                :   '0'..'7'
                )?
            )?
        |   ('4'..'7')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :   ('0'..'9')
            )?
        )
    ;
 